#include <stdio.h>
#include <readline/readline.h>
#include "atree.h"
#include "constants.h"
#include "command.h"

static const char *rcsid = "$Id: command.c,v 1.2 1994/06/14 19:03:56 queinnec Exp queinnec $";

static Tree *completion_tree;

char *treecomplete (char *text, int state)
{
    static TreeCompletion *tc = NULL;
    char *rep;
    if (state == 0) {
        TreeCompl_Success success;
        tc = tree_createcompletion (completion_tree);
        success = tree_trycompletion (tc, text);
        if (success == TREECOMPL_FAIL) {
            tree_destroycompletion (tc);
            return (NULL);
        }
    }
    rep = tree_getnextcompletion (tc);
    if (rep == NULL) {
        tree_destroycompletion (tc);
    }
    return (rep);
}

static void dolsomething (const char *cmde, const char *opt)
{
    if (rl_line_buffer[rl_end - 1] == ' ') {
        rl_end--;
        rl_line_buffer[rl_end] = '\0';
        if (rl_point > rl_end)
          rl_point = rl_end;
    }
    docommand (rl_line_buffer, cmde, opt);
}

static int dolist (int count, int key)
{
    dolsomething ("list", "-name");
    return (0);
}

static int doltrace (int count, int key)
{
    dolsomething ("ltrace", "-name");
    return (0);
}

#define METAh ('h' | 0x80)    /* This is Meta-h */

#define METAH ('H' | 0x80)    /* This is Meta-H */

/* NULL as rl_completion_entry_function means use filename completion,
 * which is different from what I want. */
static char *no_completion (char *s, int state)
{
    return (NULL);
}

void config_readline (Tree *tree,
                      char *complete (char*, int), const char *wordbreak,
                      int bind_dolist)
{
    completion_tree = tree;
    if (complete == (Function *) NULL)
      rl_completion_entry_function = (Function *)no_completion;
    else
      rl_completion_entry_function = (Function *)complete;
    if (wordbreak == NULL) {
        rl_completer_word_break_characters = (char *)"";
    } else {
        rl_completer_word_break_characters = (char *)wordbreak;
    }
    if (bind_dolist) {
        rl_bind_key (METAh, (Function *)dolist);
        rl_bind_key (METAH, (Function *)doltrace);
    } else {
        rl_unbind_key (METAh);
        rl_unbind_key (METAH);
    }
}

