/*============================================================================
 *
 *  Program: dbutils.c
 *
 *  Version: 3.0
 *
 *  Program: general database functions
 *
 *  Author:  C J Needham <cn@ibmpcug.co.uk>
 *
 *  Copyright (c) C J Needham 1994
 *
 *  OpenFile function by: Timo Lamminjoki <lamminjo@pcu.helsinki.fi>
 *
 *  Copyright (c) Timo Lamminjoki 1993
 *
 *  Permission is granted by the copyright holder to distribute this program
 *  is source form only, providing this notice remains intact, and no fee
 *  of any kind is charged. This software is provided as is and there are no 
 *  warranties, expressed or implied.
 *
 *============================================================================
 */

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include "moviedb.h"
#include "dbutils.h"

struct titleOrderRec titleOrderData [ NO_OF_FILMOGRAPHY_LISTS ] =
   { { PRDCRSRCH, "(pr)", "Producer:", "Producers:" },
     { DIRSRCH, "(d)", "Director:", "Directors:" },
     { WRITESRCH, "(w)", "Writer:", "Writers:" },
     { COMPSRCH, "(m)", "Composer:", "Composers:" },
     { CINESRCH, "(ph)", "Cinematographer:", "Cinematographers:" },
     { EDITSRCH, "(ed)", "Editor:", "Editors:" },
     { PDESSRCH, "(pd)", "Production Designer:", "Production Designers:" },
     { CDESSRCH, "(cd)", "Costume Designer:", "Costume Designers:" },
     { ACRSRCH, NULL, "Cast:", "Cast:" },
     { ACSSRCH, NULL, "Cast:", "Cast:" }
   } ;


struct filmographyOptRec filmographyOptions [ NO_OF_SEARCH_OPTS ] =
{
  {    "-a", { 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 } } ,
  {   "-ar", { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0 } } ,
  {   "-as", { 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 } } ,
  {    "-d", { 0, 0, 1, 0, 0, 0, 0, 0, 0, 0 } } ,
  {    "-w", { 0, 0, 0, 1, 0, 0, 0, 0, 0, 0 } } ,
  {    "-c", { 0, 0, 0, 0, 1, 0, 0, 0, 0, 0 } } ,
  {   "-ph", { 0, 0, 0, 0, 0, 1, 0, 0, 0, 0 } } ,
  {   "-ed", { 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 } } ,
  {   "-pd", { 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 } } ,
  {   "-cd", { 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 } } ,
  {   "-pr", { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 } } ,
  { "-name", { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 } } 
} ;



struct filmographyDefRec filmographyDefs [ NO_OF_FILMOGRAPHY_LISTS ] = 
  { { "-acr", ACRSTEM, ACRLIST, "Actor", ACRSRCH, "Acting", "Actors" } ,
    { "-acs", ACSSTEM, ACSLIST, "Actor", ACSSRCH, "Acting", "Actresses" } ,
    { "-dir", DIRSTEM, DIRLIST, "Director", DIRSRCH, "Directing", "Directors" } ,
    { "-write", WRITESTEM, WRITELIST, "Writer", WRITESRCH, "Writing", "Writers" } ,
    { "-comp", COMPSTEM, COMPLIST, "Composer", COMPSRCH, "Score", "Composers" } ,
    { "-cine", CINESTEM, CINELIST, "Cinematographer", CINESRCH, "Cinematography", "Cinematographers" } ,
    { "-edit", EDITSTEM, EDITLIST, "Editor", EDITSRCH,  "Editing", "Editors" } ,
    { "-prodes", PDESSTEM, PDESLIST, "Production Designer", PDESSRCH, "Production Design", "Production Designers" } ,
    { "-costdes", CDESSTEM, CDESLIST, "Costume Designer",  CDESSRCH, "Costume Design", "Costume Designers" } ,
    { "-prdcr", PRDCRSTEM, PRDCRLIST, "Producer", PRDCRSRCH, "Produced by", "Producers" } 
  } ;

struct triviaDefRec triviaDefs [ NO_OF_TRIV_LISTS ] = 
  { { "-triv", TRIVIA, TRIVIADB, TRIVIDX, "Trivia", "FILM TRIVIA\n" },
    { "-crazy", CRAZYLIST, CRAZYDB, CRAZYIDX, "Crazy Credits", "CRAZY CREDITS\n" },
    { "-goof", GOOFLIST, GOOFDB, GOOFIDX, "Goofs", "GOOFS LIST\n" },
    { "-quote", QUOTELIST, QUOTEDB, QUOTEIDX, "Quotes", "QUOTES LIST\n" },
    { "-strack", STRAKLIST, STRAKDB, STRAKIDX, "Soundtrack", "SOUNDTRACKS LIST\n" }
  } ;


char* duplicateString ( const char *str )
{
  char* p ;

  if ( ( p = malloc ( strlen ( str ) + 1 ) ) != NULL )
  {
     (void) strcpy ( p, str ) ;
     return ( p ) ;
  }

  moviedbError ( "error: out of memory" ) ;
  return ( NULL ) ;
}


char* duplicateField ( const char *str )
{
  char* p ;


  if ( ( p = strchr ( str, FSEP ) ) == NULL )
    return ( NULL ) ;
  else
  {
    *p = '\0' ;
    if ( ( p = malloc ( strlen ( str ) + 1 ) ) != NULL )
    {
       (void) strcpy ( p, str ) ;
       return ( p ) ;
    }
  }
  moviedbError ( "error: out of memory" ) ;
  return ( NULL ) ;
}


char *constructFilename (char *filename, const char *base, const char *extension)
{
  (void) strcpy ( filename, base ) ;
  (void) strcat ( filename, "." ) ;
  (void) strcat ( filename, extension ) ;
  return ( filename ) ;
}


int isReadable ( const char *fname )
{ 
  struct stat s ;
  char buf [ MAXPATHLEN ];

  if ( stat ( fname, &s ) == 0 )
    return ( TRUE ) ;

  (void) sprintf ( buf, "%s%s", fname, ZDBSEXT ) ;
  if ( stat ( buf, &s ) == 0 )
    return ( TRUE ) ;
    
  (void) sprintf ( buf, "%s%s", fname, ZLISTEXT ) ;
  if ( stat ( buf, &s ) == 0 )
    return ( TRUE ) ;
    
  return ( FALSE ) ;
}


int caseCompare ( char *s1, char *s2 )
{
  for ( ; upperCase (*s1) == upperCase (*s2) ; s1++, s2++ )
    if ( *s1 == '\0' )
      return ( 0 ) ;
  return ( upperCase (*s1) - upperCase (*s2) ) ;
}


void putByte ( int i, FILE *stream)
{
  (void) fputc ( i & 255, stream ) ;
}


void putTitle ( TitleID titleKey, FILE *stream)
{
  (void) fputc ( titleKey & 255, stream ) ;
  (void) fputc ( ( titleKey >> 8 ) & 255, stream ) ;
}


void putAttr ( AttributeID attrKey, FILE *stream)
{
  (void) fputc ( attrKey & 255, stream ) ;
  (void) fputc ( ( attrKey >> 8 ) & 255, stream ) ;
}


void putInt (int i, FILE *stream)
{
  (void) fputc ( i & 255, stream ) ;
  (void) fputc ( ( i >> 8 ) & 255, stream ) ;
}


void putOffset ( long offset, FILE *stream )
{
  (void) fputc ( offset & 255, stream ) ;
  (void) fputc ( ( offset >> 8 ) & 255, stream ) ;
  (void) fputc ( ( offset >> 16 ) & 255, stream ) ;
}


void putName ( NameID nameKey, FILE *stream)
{
  (void) fputc ( nameKey & 255, stream ) ;
  (void) fputc ( ( nameKey >> 8 ) & 255, stream ) ;
  (void) fputc ( ( nameKey >> 16 ) & 255, stream ) ;
}


void putString ( char *str, FILE *stream)
{
  while ( *str ) 
    (void) fputc ( *str++, stream ) ;
}


void putFilmographyCounts ( int noWithAttr, int noWithoutAttr, FILE *stream )
{
  int i ;

  i = ( ( noWithAttr & 63 ) << 10 ) | ( noWithoutAttr & 1023 ) ;
  putInt ( i, stream ) ;
}


NameID getName (FILE *stream)
{
  NameID  nameKey ;
 
  nameKey = fgetc ( stream )  & 255 ;
  nameKey |= fgetc ( stream ) << 8 ;
  nameKey |= fgetc ( stream ) << 16 ; 
  return ( nameKey ) ;
}


long getOffset (FILE *stream)
{
  long  offset ;
 
  offset = fgetc ( stream )  & 255 ;
  offset |= fgetc ( stream ) << 8 ;
  offset |= fgetc ( stream ) << 16 ; 
  return ( offset ) ;
}


int getByte (FILE *stream)
{
  return ( fgetc ( stream )  & 255 ) ;
}


TitleID getTitle (FILE *stream)
{
  TitleID  titleKey ;
 
  titleKey = fgetc ( stream )  & 255 ;
  titleKey |= fgetc ( stream ) << 8 ; 
  return ( titleKey ) ;
}


AttributeID getAttr (FILE *stream)
{
  AttributeID  attrKey ;
 
  attrKey = fgetc ( stream )  & 255 ;
  attrKey |= fgetc ( stream ) << 8 ; 
  return ( attrKey ) ;
}


int getInt (FILE *stream)
{
  int  i ;
 
  i = fgetc ( stream )  & 255 ;
  i |= fgetc ( stream ) << 8 ; 
  return ( i ) ;
}


void getFilmographyCounts ( FILE *stream, int *noWithAttr, int*noWithoutAttr )
{
  int i ;

  i = getInt ( stream ) ;
  *noWithAttr = ( i >> 10 )  & 63 ;
  *noWithoutAttr = i & 1023 ;
}


char *getString (int len, FILE *stream)
{
  char line [ MXLINELEN ] ;
  int i ;

  for ( i = 0 ; i < len ; i++ )
    line [ i ] = fgetc ( stream ) ;
  line [ i ] = '\0' ;

  return ( duplicateString ( line ) ) ;
}


void stripEOL (char *str)
{
  char *p ;

  if ( ( p = strchr ( str, '\n' ) ) != NULL )
    *p = '\0' ;
}


void stripSep (char *str)
{
  char *p ;

  if ( ( p = strchr ( str, FSEP ) ) != NULL )
    *p = '\0' ;
}


void moviedbWriteError ( const char *filename )
{
  (void) fprintf ( stderr, "error writing %s\n", filename ) ;
  exit ( -1 ) ;
}


void moviedbError ( const char *message )
{
  (void) fprintf ( stderr, "%s\n", message ) ;
  exit ( -1 ) ;
}


void moviedbUsage ( const char *s1, const char *s2, const char *s3, const char *s4 )
{
  if ( s1 != NULL )
    (void) fprintf ( stderr, "%s\n", s1 ) ;
  if ( s2 != NULL )
    (void) fprintf ( stderr, "%s\n", s2 ) ;
  if ( s3 != NULL )
    (void) fprintf ( stderr, "%s\n", s3 ) ;
  if ( s4 != NULL )
    (void) fprintf ( stderr, "%s\n", s4 ) ;
  exit ( -1 ) ;
}


void upperCaseString (char *str)
{
  while ( *str )
  {
    *str = upperCase ( *str ) ;
    str++ ;
  }
}


FILE *writeFile ( const char *path )
{
  FILE *retval ;

  if ( ( retval = fopen ( path, "w" ) ) == NULL )
    moviedbWriteError ( path ) ;

  return ( retval ) ;
}


FILE *openFile ( const char *path )
{
   FILE *ret;
#ifdef COMPRESS
   pid_t pid;
   int tmpfd;
   struct stat s ;
   char buf [ MAXPATHLEN ], tmpfname [ L_tmpnam ];
   char *args [ MAXZCATARGS ] ;
   char argstring [ MAXPATHLEN + MXLINELEN ] ;
   char *ptr ;
   int i ;
#endif

   if ( ( ret = fopen ( path, "r" ) ) == NULL )
   {
#ifndef COMPRESS
      (void) fprintf ( stderr, "Can't open %s\n", path );
      exit ( -1 );
#else
      (void) sprintf ( buf, "%s%s", path, ZDBSEXT ) ;
      if ( stat ( buf, &s ) != 0 )
      {
        (void) sprintf ( buf, "%s%s", path, ZLISTEXT ) ;
        if ( stat ( buf, &s ) != 0 )
        {
	   (void) fprintf ( stderr, "Can't find file: %s\n", path );
	   exit ( -1 );
        }
      }
      (void) tmpnam ( tmpfname );
      if ( ( tmpfd = open ( tmpfname, O_RDWR | O_CREAT ) ) == -1 )
      {
	 (void) fprintf ( stderr, "Can't open workfile\n" );
	 exit ( -1 );
      }
      /* make temporary file disappear automatically when last file descriptor
	 referencing it is closed */
      if ( unlink ( tmpfname ) )
      {
	 (void) fprintf ( stderr, "Can't unlink workfile\n" );
	 exit ( -1 );
      }
      if ( ( pid = fork () ) == -1 )
      {
	 (void) fprintf ( stderr, "Can't fork subprocess\n" );
	 exit ( -1 );
      }
      if ( pid ) /* parent */
      {
	 /* wait for child to die (and hope there were no others).
	    Should probably use waitpid. */
	 while ( wait ( NULL ) != -1 ) ;
	 lseek ( tmpfd, 0, SEEK_SET ); /* rewind uncompressed file */
	 if ( ( ret = fdopen ( tmpfd, "r+" ) ) == NULL )
	 {
	    (void) fprintf ( stderr, "Can't open workfile\n" );
	    exit ( -1 );
	 }
	 return ret;
      }
      else /* child */
      {
	 (void) dup2 ( tmpfd, 1 ); /* redirect standard output to temp file */
	 (void) close ( tmpfd ); /* close unnecessary file descriptor */
	 /* name of compressed file to buf */
         (void) sprintf ( buf, "%s%s", path, ZLISTEXT ) ;
         if ( stat ( buf, &s ) == 0 )
         {
           (void) sprintf ( argstring, "%s %s %s%s", ZLISTCAT, ZLISTCATOPTS, path, ZLISTEXT ) ;
           args [ 0 ] = argstring ;
           ptr = strtok ( argstring, " " ) ;
           for ( i = 1 ; ptr != NULL ; i++ )
           {
             ptr = strtok ( NULL, " " ) ;
             args [ i ] = ptr ;
           }
	   execvp ( args [ 0 ], args );
	   (void) fprintf ( stderr, "execvp failed\n" );
	   exit ( -1 );
         }
         else
         {
           (void) sprintf ( argstring, "%s %s %s%s", ZDBSCAT, ZDBSCATOPTS, path, ZDBSEXT ) ;
           args [ 0 ] = argstring ;
           ptr = strtok ( argstring, " " ) ;
           for ( i = 1 ; ptr != NULL ; i++ )
           {
             ptr = strtok ( NULL, " " ) ;
             args [ i ] = ptr ;
           }
	   execvp ( args [ 0 ], args );
	   (void) fprintf ( stderr, "execvp failed\n" );
	   exit ( -1 );
         }
      }
#endif /* COMPRESS */
   }
   else
     return ret ;
   return ( NULL ) ;
}


FILE *copyFile ( const char *toName, const char *fromName ) 
{
  FILE *toFp, *fromFp ;
  int c ;

  fromFp = openFile ( fromName ) ;
  toFp = writeFile ( toName ) ;

  if ( ( toFp = fopen ( toName, "w+" ) ) == NULL )
    moviedbError ( "error: can't open temporary file" ) ;

  while ( ( c = fgetc ( fromFp ) ) != EOF )
    fputc ( c, toFp ) ;
  (void) fclose ( fromFp ) ;
  
  if ( unlink ( toName ) )
    moviedbError ( "error: can't unlink temporary file" ) ;
  rewind ( toFp ) ;
  return ( toFp ) ;
}


long findSOL ( FILE *stream, long pos )
{
   int ch;

   if ( pos > 0 )
   {
      do {
         pos--;
         (void) fseek ( stream, pos, 0 ) ;
         ch = fgetc ( stream ) ;
      } while ( (ch != '\n') && (ch != EOF) && (pos > 0) ) ;
      if ( pos != 0 )
          pos++ ;
      else
           (void) fseek ( stream, pos, 0 ) ;
   }
   else
   {
       pos = 0 ;
       (void) fseek ( stream, pos, 0 ) ;
   }
   return pos ;
}


int fieldCaseCompare ( char *s1, char *s2 )
{
  if ( s2 == NULL )
    return ( 1 ) ;

  while ( upperCase  ( *s1 ) == upperCase  ( *s2 ) )
  {
    if ( *(s1+1) == FSEP && *(s2+1) == '\0' )
      return ( 0 ) ;
    s1++; s2++ ;
  }

  if ( *s1 == FSEP )
    return ( -1 ) ;
  else if ( *s2 == '\0' )
    return ( 1 ) ;
  else
    return ( upperCase ( *s1 ) - upperCase  ( *s2 ) ) ;
}


struct titleSearchRec *newTitleSearchRec ( void )
{
  struct titleSearchRec *rrec ;
  int i ;

  if ( ( rrec = (struct titleSearchRec *) malloc ( sizeof ( struct titleSearchRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;

  rrec -> title = NULL ;
  rrec -> titleKey = NOTITLE ;
  rrec -> nameCount = 0 ;
  rrec -> year = 0 ;
  rrec -> attrKey = NOATTR ;
  rrec -> attr = NULL ;
  rrec -> time = NULL ;
  rrec -> mrr = NULL ;
  rrec -> aka = NULL ;
  rrec -> plot = NULL ;
  rrec -> next = NULL ;
    
  for ( i = 0 ; i < MAXTITLERESULTS ; i++ )
  {
    rrec -> entries [ i ] . nameKey = NONAME ;
    rrec -> entries [ i ] . attrKey = NOATTR ;
    rrec -> entries [ i ] . name = NULL ;
    rrec -> entries [ i ] . attr = NULL ;
    rrec -> entries [ i ] . cname = NULL ;
  }

  for ( i = 0 ; i < NO_OF_FILMOGRAPHY_LISTS ; i++ )
  {
    rrec -> results [ i ] . count = 0 ;
    rrec -> results [ i ] . base = -1 ;
  }

  for ( i = 0 ; i < NO_OF_TRIV_LISTS ; i++ )
  {
    rrec -> trivflags [ i ] = FALSE ;
    rrec -> trivlists [ i ] = NULL ;
  }

  return ( rrec ) ;
}


struct nameSearchRec *newNameSearchRec ( void )
{
  struct nameSearchRec *rrec ;
  int i ;

  if ( ( rrec = (struct nameSearchRec *) malloc ( sizeof ( struct nameSearchRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;

  rrec -> name = NULL ;
  rrec -> nameKey = NONAME ;
  rrec -> firstMatch = -1 ;
  rrec -> biography = NULL ;
  rrec -> next = NULL ;
  for ( i = 0 ; i < NO_OF_FILMOGRAPHY_LISTS ; i++ )
  {
    rrec -> lists [ i ] = NULL ;
    rrec -> searchFlags [ i ] = FALSE ;
  }
  return ( rrec ) ;
}


struct plotRec *newPlotRec ( void )
{
  struct plotRec *retval ;

  if ( ( retval = (struct plotRec *) malloc ( sizeof ( struct plotRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct outlineRec *newOutlineRec ( void )
{
  struct outlineRec *retval ;

  if ( ( retval = (struct outlineRec *) malloc ( sizeof ( struct outlineRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct mrrRec *newMrrRec ( void )
{
  struct mrrRec *retval ;

  if ( ( retval = (struct mrrRec *) malloc ( sizeof ( struct mrrRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct listRec *newListRec ( void )
{
  struct listRec *retval ;

  if ( ( retval = (struct listRec *) malloc ( sizeof ( struct listRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct akaRec *newAkaRec ( void )
{
  struct akaRec *retval ;

  if ( ( retval = (struct akaRec *) malloc ( sizeof ( struct akaRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct timeRec *newTimeRec ( void )
{
  struct timeRec *retval ;

  if ( ( retval = (struct timeRec *) malloc ( sizeof ( struct timeRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct personRec *newPersonRec ( void )
{
  struct personRec *retval ;

  if ( ( retval = (struct personRec *) malloc ( sizeof ( struct personRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct lineRec *newLineRec ( void )
{
  struct lineRec *retval ;

  if ( ( retval = (struct lineRec *) malloc ( sizeof ( struct lineRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct bioRec *newBioRec ( void )
{
  struct bioRec *retval ;

  if ( ( retval = (struct bioRec *) malloc ( sizeof ( struct bioRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


struct bioMiscRec *newBioMiscRec ( void )
{
  struct bioMiscRec *retval ;

  if ( ( retval = (struct bioMiscRec *) malloc ( sizeof ( struct bioMiscRec ) ) ) == NULL )
    moviedbError ( "error: out of memory" ) ;
    
  return ( retval ) ;
}


void displayTitleYear ( char *title, int year ) 
{
   char *bptr ;

   if ( ( bptr = strchr ( title, '(' ) ) != NULL )
   {
     if ( *(bptr+1) == '1' && *(bptr+2) == '9' )
       (void) printf ( "%s", title ) ;
     else 
     {
       *bptr = '\0' ;
       if ( year > 0 )
         (void) printf ( "%s(%4d) (%s", title, year, bptr + 1) ;
       else
         (void) printf ( "%s(\?\?\?\?) (%s", title, bptr + 1 ) ;
       *bptr = '(' ;
     }
   }
   else
     if ( year > 0 )
       (void) printf ( "%s (%4d)", title, year ) ;
     else
       (void) printf ( "%s (\?\?\?\?)", title ) ;
}


char *caseStrStr ( char *line, char *str ) 
{
  char upperLine [ MXLINELEN ] ;
  char upperStr [ MXLINELEN ] ;
  char *from, *to ;

  for ( from = line, to = upperLine ; *from ; from++, to++ )
    if ( islower ( *from ) )
      *to = toupper ( *from ) ;
    else
      *to = *from ;
  *to = '\0' ;

  for ( from = str, to = upperStr ; *from ; from++, to++ )
    if ( islower ( *from ) )
      *to = toupper ( *from ) ;
    else
      *to = *from ;
  *to = '\0' ;

  return ( strstr ( upperLine, upperStr ) ) ;
}


char *mapNameKeyToText ( NameID nameKey, FILE *nameKeyFp, FILE *nameIndexFp )
{
  char line [ MXLINELEN ] ;
  long offset ;

  (void) fseek ( nameIndexFp, 3 * nameKey, SEEK_SET ) ;
  offset = getOffset ( nameIndexFp ) ;
  (void) fseek ( nameKeyFp, offset, SEEK_SET ) ;
  (void) fgets ( line, MXLINELEN, nameKeyFp ) ;
  return ( duplicateField ( line ) ) ;
}


char *mapTitleKeyToText ( TitleID titleKey, FILE *titleKeyFp, FILE *titleIndexFp )
{
  char line [ MXLINELEN ] ;
  long offset ;

  (void) fseek ( titleIndexFp, 3 * titleKey, SEEK_SET ) ;
  offset = getOffset ( titleIndexFp ) ;
  (void) fseek ( titleKeyFp, offset, SEEK_SET ) ;
  (void) fgets ( line, MXLINELEN, titleKeyFp ) ;
  return ( duplicateField ( line ) ) ;
}


char *mapAttrKeyToText ( AttributeID attrKey, FILE *attrKeyFp, FILE *attrIndexFp )
{
  char line [ MXLINELEN ] ;
  long offset ;

  if ( attrKey != NOATTR )
  {
    (void) fseek ( attrIndexFp, 3 * attrKey, SEEK_SET ) ;
    offset = getOffset ( attrIndexFp ) ;
    (void) fseek ( attrKeyFp, offset, SEEK_SET ) ;
    (void) fgets ( line, MXLINELEN, attrKeyFp ) ;
    return ( duplicateField ( line ) ) ;
  }
  else
    return ( NULL ) ;
}


char *mapfastNameKeyToText ( NameID nameKey, FILE *nameKeyFp, FILE *nameIndexFp )
{
  static char line [ MXLINELEN ] ;
  long offset ;
  char *p;

  (void) fseek ( nameIndexFp, 3 * nameKey, SEEK_SET ) ;
  offset = getOffset ( nameIndexFp ) ;
  (void) fseek ( nameKeyFp, offset, SEEK_SET ) ;
  (void) fgets ( line, MXLINELEN, nameKeyFp ) ;
  if ( (p = strchr ( line, FSEP ) ) == NULL )
    return ( NULL ) ;
  *p = '\0' ;
  return ( line ) ;
}


void displayFormatPlain ( struct formatRec listData [], int count )
{
  int i, len ;

  if ( count < 1 )
    return ;

  (void) printf ( "%s", listData [ 0 ] . name ) ;
  len = strlen ( listData [ 0 ] . name ) ;
  if ( len >= 16 )
    (void) printf ( "\t" ) ;
  else if ( len >= 8 )
    (void) printf ( "\t\t" ) ;
  else
    (void) printf ( "\t\t\t" ) ;
    
  (void) printf ( "%s", listData [ 0 ] . title ) ;
  if ( listData [ 0 ] . attr != NULL )
    (void) printf ( " %s\n", listData [ 0 ] . attr ) ;
  else
    (void) printf ( "\n" ) ;

  for ( i = 1 ; i < count ; i++ )
  {
    (void) printf ( "\t\t\t" ) ;
    (void) printf ( "%s", listData [ i ] . title ) ;
    if ( listData [ i ] . attr != NULL )
      (void) printf ( " %s\n", listData [ i ] . attr ) ;
    else
      (void) printf ( "\n" ) ;
  }

  (void) printf ( "\n" ) ;
}


void freeFormatData ( struct formatRec listData [], int count )
{
  int i ;

  for ( i = 0 ; i < count ; i++ )
  {
    free ( (void*) listData [ i ] . name ) ;
    free ( (void*) listData [ i ] . title ) ;
    free ( (void*) listData [ i ] . attr ) ;
  }
}
