/*============================================================================
 *
 *  Program: moviedb.h
 *
 *  Version: 3.0
 *
 *  Purpose: general constant and type definitions for movie database
 *
 *  Author:  C J Needham <cn@ibmpcug.co.uk>
 *
 *  Copyright (c) C J Needham 1994
 *
 *  Permission is granted by the copyright holder to distribute this program
 *  is source form only, providing this notice remains intact, and no fee
 *  of any kind is charged. This software is provided as is and there are no
 *  warranties, expressed or implied.
 *
 *============================================================================
 */

#define   MXLINELEN               300
#define   FSEP                    '|'
#define   MAXZCATARGS              10
#define   MAXLSEEN              50000
#define   MAXTITLES             50000
#define   MAXATTRS               2500
#define   MAXNAMES             120000
#define   MAXFILMOGRAPHIES        511
#define   MAXWITHATTRS             63
#define   MAXTITLERESULTS         511
#define   MAXALISTRESULTS        5000
#define   MAXALISTTITLES         3000
#define   MAXLISTNAMES          70000
#define   MAXLISTENTRIES       200000
#define   NOTITLE              0xffff
#define   NOATTR               0xffff
#define   NONAME           0xffffffff
#define   MAXTRIVENTRIES         2000
#define   MAXPLOTENTRIES         4000
#define   MAXBIOENTRIES         10000
#define   MAXMRRCOR               100
#define   MAXMRRENTRIES          8000
#define   MAXAKAENTRIES         16000

#ifndef DBDIR
#define   DBDIR
#define   RAWDIR
#define   ETCDIR
#endif

#define DBSEXT  "data"
#define NDXEXT  "names"
#define TDXEXT  "titles"

#define   MRRDB       DBDIR "ratings.data"
#define   AKADB       DBDIR "aka.data"
#define   MOVIEDB     DBDIR "movies.data"
#define   TIMESDB     DBDIR "running-times.data"
#define   PLOTDB      DBDIR "plot.data"
#define   BIODB       DBDIR "biographies.data"
#define   VOTEDB      DBDIR "votes.data"

#define   TRIVIADB    DBDIR "trivia.data"
#define   CRAZYDB     DBDIR "crazy-credits.data"
#define   QUOTEDB     DBDIR "quotes.data"
#define   STRAKDB     DBDIR "soundtracks.data"
#define   GOOFDB      DBDIR "goofs.data"

#define   ACRSTEM     DBDIR "actors"
#define   ACSSTEM     DBDIR "actresses"
#define   DIRSTEM     DBDIR "directors"
#define   WRITESTEM   DBDIR "writers"
#define   COMPSTEM    DBDIR "composers"
#define   CINESTEM    DBDIR "cinematographers"
#define   EDITSTEM    DBDIR "editors"
#define   PDESSTEM    DBDIR "production-designers"
#define   CDESSTEM    DBDIR "costume-designers"
#define   PRDCRSTEM   DBDIR "producers"

#define   TITLEIDX  DBDIR "titles.index"
#define   ATTRIDX   DBDIR "attributes.index"
#define   NAMEIDX   DBDIR "names.index"

#define   TITLEKEY  DBDIR "titles.key"
#define   ATTRKEY   DBDIR "attributes.key"
#define   NAMEKEY   DBDIR "names.key"

#define   KEYCOUNTS DBDIR "key.counts"

#define   BIOIDX    DBDIR "biographies.index"
#define   AKAIDX    DBDIR "aka.index"
#define   PLOTIDX   DBDIR "plot.index"

#define   CRAZYIDX  DBDIR "crazy-credits.index"
#define   TRIVIDX   DBDIR "trivia.index"
#define   QUOTEIDX  DBDIR "quotes.index"
#define   STRAKIDX  DBDIR "soundtracks.index"
#define   GOOFIDX   DBDIR "goofs.index"

#define   MRRCOR    ETCDIR "corrects.mrr"
#define   BIODEFS   ETCDIR "biodefs"

#define   ACRLIST      RAWDIR "actors.list"
#define   ACSLIST      RAWDIR "actresses.list"
#define   DIRLIST      RAWDIR "directors.list"
#define   WRITELIST    RAWDIR "writers.list"
#define   COMPLIST     RAWDIR "composers.list"
#define   CINELIST     RAWDIR "cinematographers.list"
#define   MRRLIST      RAWDIR "ratings.list"
#define   AKALIST      RAWDIR "aka-titles.list"
#define   TRIVIA       RAWDIR "trivia.list"
#define   PLOTLIST     RAWDIR "plot.list"
#define   CRAZYLIST    RAWDIR "crazy-credits.list"
#define   BIOLIST      RAWDIR "biographies.list"
#define   EDITLIST     RAWDIR "editors.list"
#define   PDESLIST     RAWDIR "production-designers.list"
#define   CDESLIST     RAWDIR "costume-designers.list"
#define   MOVIELIST    RAWDIR "movies.list"
#define   TIMESLIST    RAWDIR "running-times.list"
#define   VOTELIST     RAWDIR "votelog"
#define   QUOTELIST    RAWDIR "quotes.list"
#define   STRAKLIST    RAWDIR "soundtracks.list"
#define   GOOFLIST     RAWDIR "goofs.list"
#define   PRDCRLIST    RAWDIR "producers.list"

#define   TRUE      1
#define   FALSE     0

#define   NO_OF_FILMOGRAPHY_LISTS   10
#define   NO_OF_CAST_LISTS           2
#define   NO_OF_CHAR_LISTS           2
#define   NO_OF_SEARCH_OPTS         12
#define   GLOBAL_NAME_SEARCH        11
#define   NO_OF_TRIV_LISTS           5

#define   ACRSRCH     0
#define   ACSSRCH     1
#define   DIRSRCH     2
#define   WRITESRCH   3
#define   COMPSRCH    4
#define   CINESRCH    5
#define   EDITSRCH    6
#define   PDESSRCH    7
#define   CDESSRCH    8
#define   PRDCRSRCH   9

#define   NONE   0
#define   MRR    1
#define   SMRR   2
#define   VMRR   3

#define   YR     1
#define   YRU    2

#define  MV_STATE        0
#define  PL_STATE        1

#define  TITLE_KEY    "MV"
#define  PLOT_KEY     "PL"
#define  BY_KEY       "BY"
#define  GENRE_KEY    "GE"
#define  REVIEW_KEY   "RV"

#define  PERSON_STATE    0
#define  BIO_STATE       1

#define  NAME_KEY       "NM"
#define  REALNAME_KEY   "RN"
#define  BIRTHDATE_KEY  "DB"
#define  DEATHDATE_KEY  "DD"
#define  BIO_KEY        "BG"
#define  AUTHOR_KEY     "BY"

#define  MAXBIODEFS      25

#define  TRIVTITLE "# " 

#include <sys/param.h>
#ifndef MAXPATHLEN
#  define MAXPATHLEN 300
#endif

#ifndef SEEK_SET
#  define SEEK_SET 0
#  define SEEK_CUR 1
#  define SEEK_END 2
#endif


typedef unsigned int AttributeID ;
typedef unsigned int TitleID ;
typedef unsigned long NameID ;


struct personRec
{
  struct bioRec     *biography ;
  char              *RN ;
  char              *DB ;
  char              *DD ;
  struct bioMiscRec *misc ;
} ;


struct bioRec 
{
  struct lineRec *biog ;
  char            *BY ;
  struct bioRec  *next ;
} ;


struct bioMiscRec
{
  struct lineRec  *list ;
  char             label [ 2 ] ;
  struct bioMiscRec *next ;
} ;

 
struct outlineRec
{
  struct lineRec *PL ;
  char *BY ;
  struct outlineRec *next ;
} ;


struct lineRec 
{
  char            *text ;
  struct lineRec *next ;
}  ;


struct plotRec 
{
  struct outlineRec *outline ;
  struct lineRec    *GE ;
  char               *RV ;
  struct lineRec    *misc ;
} ;


struct triviaRec  
{ 
  int             count ;
  struct lineRec *trivia ;  
} ;


struct titleOrderRec 
{
  int  listId ;
  char *tag ;
  char *description ;
  char *pluralDesc ;
} ;


struct filmographyOptRec
{
  char   *option ;
  int    listflags [ NO_OF_FILMOGRAPHY_LISTS ] ;
} ;


struct filmographyDefRec
{
  char   *option ;
  char   *stem ;
  char   *list ;
  char   *filmography ;
  int    listid ;
  char   *class ;
  char   *desc ;
} ;


struct triviaDefRec
{
  char   *option ;
  char   *list ;
  char   *db ;
  char   *index ;
  char   *desc ;
  char   *start ;
} ;


struct mrrRec
{
  char   distribution [11] ;
  int    votes ;
  float  rating ;
} ;


struct timeRec
{
  AttributeID    attrKey ;
  int            runningTime ;
  char           *attr ;
  struct timeRec *next ;
} ;


struct akaRec
{
  TitleID  akaKey ;
  char *akaTitle ;
  struct akaRec *next ;
} ;


struct titleResults
{
  int  count ;
  int  base ;
} ;


struct titleListEntry
{
  NameID nameKey ;
  AttributeID  attrKey ;
  char *name ;
  char *attr ;
  char *cname ;
} ;


struct titleSearchOptRec
{
  unsigned int casesen : 1 ;
  unsigned int akaopt : 1 ;
  unsigned int chopt : 1 ;
  unsigned int mrropt : 1 ;
  unsigned int yropt : 1 ;
  unsigned int plotopt : 1 ;
  unsigned int substring : 1 ;
  unsigned int timeOpt : 1 ;
} ;


struct titleSearchRec
{
  char                     *title ;
  TitleID                  titleKey ;
  struct titleSearchOptRec searchparams ;
  int                      trivflags [ NO_OF_TRIV_LISTS ] ; 
  int                      nameCount ;
  struct titleResults      results [ NO_OF_FILMOGRAPHY_LISTS ] ;
  struct titleListEntry    entries [ MAXTITLERESULTS ] ;
  struct lineRec           *trivlists [ NO_OF_TRIV_LISTS ] ; 
  int                      year ; 
  AttributeID              attrKey ;
  char                     *attr ;
  struct mrrRec            *mrr ;
  struct timeRec           *time ;
  struct akaRec            *aka ;
  struct plotRec           *plot ; 
  struct titleSearchRec    *next ;
} ;


struct listEntry
{
  TitleID       titleKey ;
  AttributeID   attrKey ;
  char          *title ;
  char          *attr ;
  char          *cname ;
  int           year ;
  struct akaRec *aka ;
  struct mrrRec *mrr ;
} ;


struct listRec
{
   int              count ;
   struct listEntry entries [ MAXFILMOGRAPHIES ] ;
} ;


struct nameSearchOptRec
{
  unsigned int approx : 4 ;
  unsigned int mrropt : 2 ;
  unsigned int yropt : 2 ;
  unsigned int searchall : 1 ;
  unsigned int casesen : 1 ;
  unsigned int substring : 1 ;
  unsigned int mvsonly : 1 ;
  unsigned int chopt : 1 ;
  unsigned int biopt : 1 ;
  unsigned int akaopt : 1 ;
} ;


struct nameSearchRec
{
  char                    *name ;
  NameID                  nameKey ;
  struct nameSearchOptRec searchparams ;
  int                     searchFlags [ NO_OF_FILMOGRAPHY_LISTS ] ;
  int                     firstMatch ;
  struct listRec          *lists  [ NO_OF_FILMOGRAPHY_LISTS ] ;
  struct personRec        *biography ;
  struct nameSearchRec    *next ;
} ;


struct attrSearchOptRec
{
  unsigned int string : 1 ;
  unsigned int aa : 1 ;
  unsigned int aan : 1 ;
  unsigned int gg : 1 ;
  unsigned int ggn : 1 ;
} ;


struct alistTitleRes
{
  char *title ;
  char *attr ;
  int  year ;
} ;


struct alistResult
{
  char *name ;
  char *title ;
  char *attr ;
  int  year ;
} ;


struct titleDbRec
{
  TitleID  titleKey ;
  int      year ;
} ;


struct mrrDbRec
{
  TitleID       titleKey ;
  struct mrrRec mrr ;
} ; 


struct voteDbRec
{
  TitleID titleKey ;
  int  vote ;
} ;


struct lindexRec
{
  FILE   *fp ;
  int    listId ;
  char   name [ MXLINELEN ] ;
} ;


struct lindexTitleRec
{
  char           *title ;
  struct mrrRec  mrr ;
  int            year ;
} ; 


struct formatRec
{
   char          *name ;
   char          *title ;
   char          *attr ;
   NameID        nameKey ;
   TitleID       titleKey ;
   AttributeID   attrKey ;
   int           year ;
   struct mrrRec mrr ;
} ; 


struct lseenCount
{
   NameID nameKey ;
   char   *name ;
   int    count ;
   long   offset ;
} ;



struct titleIndexRec
{
  char     *title ;
  TitleID  titleKey ;
} ;


struct attrIndexRec
{
  char        *attr ;
  AttributeID attrKey ;
} ;


struct nameIndexRec
{
  char  *name ;
  NameID  nameKey ;
} ;


struct attrKeyOffset
{
  AttributeID  attrKey ;
  long         offset ;
} ;


struct titleKeyOffset
{
  TitleID  titleKey ;
  long     offset ;
} ;


struct nameKeyOffset
{
  NameID  nameKey ;
  long    offset ;
} ;


struct titleAttrPair
{
  TitleID     titleKey ;
  AttributeID attrKey ;
} ;


struct filmography 
{
  NameID               nameKey ;
  int                  noWithAttr ;
  int                  noWithoutAttr ;
  struct titleAttrPair withAttrs [ MAXFILMOGRAPHIES ] ;
  TitleID              withoutAttrs [ MAXFILMOGRAPHIES ] ;
} ;


struct titleCharPair
{
  TitleID     titleKey ;
  char        *cname ;
} ;

struct titleAttrChar
{
  TitleID     titleKey ;
  AttributeID attrKey ;
  char        *cname ;
} ;


struct castFilmography 
{
  NameID               nameKey ;
  int                  noWithAttr ;
  int                  noWithoutAttr ;
  struct titleAttrChar withAttrs [ MAXFILMOGRAPHIES ] ;
  struct titleCharPair withoutAttrs [ MAXFILMOGRAPHIES ] ;
} ;


struct corrections 
{
  char  *original ;
  char  *corrected ;
} ;


struct mrrData
{
  TitleID  titleKey ;
  char     distribution[11] ;
  int      votes ;
  int      rating ;
} ;


struct timeData
{
  TitleID      titleKey ;
  int          time ;
  AttributeID  attrKey ;
} ;


struct yearData
{
  TitleID     titleKey ;
  int         yrfrom ;
  int         yrto ;
  AttributeID attrKey ;
} ;


struct voteData
{
  TitleID   titleKey ;
  int       vote ;
} ;


struct akaData
{
  int   primary ;
  int   aka ;
} ;
