#include <stdio.h>
#include <string.h>

#include "moviedb.h"
#include "dbutils.h"

#include "atree.h"
#include "constants.h"
#include "readname.h"

static const char *rcsid = "$Id: readname.c,v 1.9 1994/05/20 14:25:25 queinnec Exp $";

int readname_keyfile (Tree *tree, const char *filename)
{
    FILE *f;
    char line[MXLINELEN];
    char *p;
    int linenum = 0;
    
    f = openFile (filename);
    if (f == NULL) {
        fprintf (stderr, "Can't open %s: ", filename);
        perror ("");
        return (ERROR);
    }
    fprintf (stderr, "Loading %s.", filename);
    while (1) {
        (void) fgets (line, MXLINELEN, f) ;
        if  ((p = strchr (line, FSEP)) == NULL)
          break;
        *p = 0;
        /* assert (! tree_check (tree, line)); */
        if (tree_insert (tree, line) == ERROR)
          return (ERROR);
        linenum++;
        if (linenum % 1000 == 0)
          fprintf (stderr, ".");
    }
    fprintf (stderr, "%d names inserted.\n", linenum);
    fclose (f);
    return (OK);
}

int readname_dbs (Tree *tree, const char *dbsstem)
{
    FILE *indexfp;
    FILE *namekeyfp;
    FILE *nameidxfp;
    char dbs[MAXPATHLEN];
    long namekey;
    int linenum = 0;
    const char *name;

    sprintf (dbs, "%s.%s", dbsstem, NDXEXT);
    indexfp = openFile (dbs);
    namekeyfp = openFile (NAMEKEY);
    nameidxfp = openFile (NAMEIDX);
    fprintf (stderr, "Loading %s.", dbs);
    while (! feof(indexfp)) {
        namekey = getName (indexfp);
        if (namekey == -1)
          break;
        (void) getOffset (indexfp) ;
        name = mapfastNameKeyToText (namekey, namekeyfp, nameidxfp);
        if (tree_insert (tree, name) == ERROR)
          return (ERROR);
        linenum++;
        if (linenum % 1000 == 0)
          fprintf (stderr, ".");
    }
    fprintf (stderr, "%d names.\n", linenum);
    fclose (indexfp);
    fclose (namekeyfp);
    fclose (nameidxfp);
    return (OK);
}
