#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include "constants.h"
#include "rectify.h"

static const char *rcsid = "$Id: rectify.c,v 1.2 1994/07/01 11:54:54 queinnec Exp queinnec $";

/*
 * This should be coming from dbutils.c, but I prefer rectify.c to be
 * independent.
 */
static char *duplicateString (const char *s)
{
    char *r;
    if (s == NULL)
      return (NULL);
    r = (char *) malloc (strlen (s) + 1);
    if (r == NULL)
      return (NULL);
    strcpy (r, s);
    return (r);
}

/*
 * Just like strstr(), but I am sick of systems without it.
 * strlen (s2) must be >= 1 (I don't know what strstring ("xxx", "") does)
 */
static const char *strstring (const char *s1, const char *s2)
{
    int s2len = strlen (s2);
    for (; *s1 != '\0'; s1++) {
        if (strncmp (s1, s2, s2len) == 0)
          break;
    }
    if (*s1 != '\0')
      return (s1);
    else
      return (NULL);
}

/*
 * These are strings which looks like attributes but are not.
 * For instance, "williams, john (ii) (AAN)" must become
 *  "John Williams (ii) (AAN)", but the attribute is only "(AAN)".
 */
static const char *attr_exceptions[] = {
    "(ii)", "(iii)", "(iv)", "(v)",
    NULL
};

/*
 * Translate from "surname, firstname" into "firstname surname", and then
 * capitalize every word.
 * Special care is given to attributes such as (AA).
 * Even more special care is given to false attributes such as (ii).
 * If NAME is of the form: xxx..yyyy, yyyy is assumed to be the played
 *  character name and is removed to get the cleaned name.
 * The attributes must come *before* the character.
 *
 * The worst line is:
 * williams, john (ii) (AAN) (GG) ...... Eeek Oook (ii) (not a monkey)
 *
 * Four results:
 *  - RCLEANEDNAME holds the cleaned name (without attribute)
 *  - RNEWNAME holds the reversed name (without attribute)
 *  - RATTRIBUTE holds the attribute or NULL if none.
 *  - RCHARNAME holds the character name or NULL if none.
 */
void rectify (const char *name,
              char **rcleanedname, char **rnewname,
              char **rattribute, char **rcharname)
{
    char *newname;
    char *cleanedname;
    char *falseattr = NULL;
    char *attr;
    char *p;
    const char *charname;
    int docapit;

    cleanedname = duplicateString (name);

    /* grab the character name, and removed it from cleanedname. */
    charname = strstring (name, "..");
    if (charname != NULL) {
        char *r;
        while ((*charname == '.') || (*charname == ' '))
          charname++;
        if (*charname == '\0')  /* "actor name ...." without charname */
          charname = NULL;
        /* Remove useless blanks. */
        r = strstring (cleanedname, "..");
        r--;
        while ((r >= cleanedname) && (*r == ' '))
          r--;
        r++;
          /* protection against ".... charname" without actor name */
        if (r != cleanedname)
          *r = '\0';
    }
        
    /* find the attribute part */
    attr = cleanedname;
  silly_john_williams:          /* John Williams (ii) can also win (AA) */
    attr = strchr (attr, '(');
    if (attr != NULL) {
        const char **except;
        /* ignore (ii) and co */
        for (except = attr_exceptions; *except != NULL; except++) {
            if (strncmp (attr, *except, strlen (*except)) == 0) {
                if (falseattr == NULL)
                  falseattr = attr; /* could there be *2* false attributes? */
                attr++;
                goto silly_john_williams;
            }
        }
        /* ignore blanks before the attribute */
        if (attr != cleanedname) { /* beware of "(AAN)" alone! */
            for (p = attr - 1; (p != cleanedname) && (*p == ' '); p--)
              /* empty */;
            *(p+1) = '\0';
        }
    }

    /* Reverse "surname, firstname" into "firstname surname" */
    p = strchr (cleanedname, ',');
    if (p == NULL) {
        /* "foo": no comma in name */
        newname = duplicateString (cleanedname);
    } else if (*(p+1) == '\0') {
        /* "foo," */
        newname = duplicateString (cleanedname);
    } else {
        int surnamelen;
        char *q;
        newname = (char *) malloc (strlen (cleanedname) + 1);
        if (*(p+1) != ' ')      /* "foo,bar" */
          q = p+1;
        else                    /* "foo, bar" at least */
          q = p+2;
        surnamelen = strlen (q) -
          (falseattr == NULL ? 0 : (strlen (falseattr) + 1));
        strncpy (newname, q, surnamelen);
        newname[surnamelen] = 0;
        strcat (newname, " ");
        strncat (newname, cleanedname, strlen (cleanedname) - strlen (p));
        if (falseattr != NULL) {
            strcat (newname, " ");
            strcat (newname, falseattr);
        }
    }

    /* Capitalize words */
    for (docapit = TRUE, p = newname; *p != '\0'; p++) {
        if (docapit) {
            docapit = FALSE;
            if (islower (*p))
              *p = toupper (*p);
        }
        if (*p == ' ')
          docapit = TRUE;
    }

    *rnewname = newname;
    *rcleanedname = cleanedname;
    if (attr != NULL) 
      *rattribute = duplicateString (attr);
    else 
      *rattribute = NULL;
    if (charname != NULL)
      *rcharname = duplicateString (charname);
    else
      *rcharname = NULL;
}

