#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include "constants.h"
#include "y-or-n.h"
#include <readline/readline.h>

static const char *rcsid = "$Id: y-or-n.c,v 1.1 1994/06/28 07:35:59 queinnec Exp queinnec $";

/*
 * Readline 1.1 doesn't have get_keymap/set_keymap and uses keymap as the
 *  standard keymap.
 * Readline 2.0 (from bash) has the functions and uses _rl_keymap.
 * There is no switch or define available to find the version, except
 * something as stupid as the following...
 */
#ifndef KEYMAP_SIZE             /* assumed version 1.1 */

extern Keymap keymap;

static Keymap rl_get_keymap (void)
{
    return (keymap);
}
static void rl_set_keymap (Keymap map)
{
   keymap = map;
}

#endif /* KEYMAP_SIZE */


static int yes_or_no;

static void y_or_n_yes (int count, int key)
{
    yes_or_no = TRUE;
    rl_insert (1, 'y');
    rl_redisplay ();
    rl_newline (1, '\010');
}

static void y_or_n_no (int count, int key)
{
    yes_or_no = FALSE;
    rl_insert (1, 'n');
    rl_redisplay ();
    rl_newline (1, '\010');
}

int y_or_n_p (const char *s)
{
    static int initdone = FALSE;
    static Keymap y_or_n_keymap;
    static const char *stdprompt = "(y or n) ";
    Keymap savedkeymap;
    char *prompt;
    char *line;
    if (! initdone) { 
        initdone = TRUE;
        y_or_n_keymap = rl_make_bare_keymap ();
        rl_bind_key_in_map ('y', (Function *) y_or_n_yes, y_or_n_keymap);
        rl_bind_key_in_map (' ', (Function *) y_or_n_yes, y_or_n_keymap);
        rl_bind_key_in_map ('n', (Function *) y_or_n_no, y_or_n_keymap);
        rl_bind_key_in_map (RUBOUT, (Function *) y_or_n_no, y_or_n_keymap);
    }
    yes_or_no = FALSE;
    savedkeymap = rl_get_keymap ();
    rl_set_keymap (y_or_n_keymap);
    if (s != NULL) {
        prompt = (char *) malloc (strlen (s) + strlen (stdprompt) + 1);
        strcpy (prompt, s);
        strcat (prompt, " ");
        strcat (prompt, stdprompt);
    } else {
        prompt = (char *) malloc (strlen (stdprompt) + 1);
        strcpy (prompt, stdprompt);
    }
    line = readline (prompt);
    if (line != NULL)
      free (line);
    free (prompt);
    rl_set_keymap (savedkeymap);
    return (yes_or_no);
}
